require "rampart/request"
require "rampart/events/fail_event"

module Rampart
  class StrategyManager
    def initialize(env)
      @env = env
      @strategies = {}
    end

    def register(name, strategy)
      raise(ArgumentError, "strategy name already registered: '#{name}'") if @strategies.key?(name)
      @strategies[name] = strategy
    end

    def authenticate!(*names)
      winning_event = FailEvent.new
      request = Request.new(@env)
      names.each do |name|
        strategy = fetch(name)
        next unless strategy.valid?(request)
        event = strategy.authenticate!(request)
        if event.halt?
          winning_event = event
          break
        end
      end
      winning_event
    end

    private

    def fetch(name)
      raise(ArgumentError, "Strategy not registered: '#{name}'") unless @strategies.key?(name)
      @strategies[name]
    end
  end
end